#include <bcm2835.h>  
#include <stdint.h>
#include <stdio.h>  

#define SPI_HOLDTIMEUS	5

const uint8_t gpio_pin_cs	= 36;	// Define pins here as a const. TODO: variable pins based on cmdline arguments?
const uint8_t gpio_pin_clk	= 39;
const uint8_t gpio_pin_mosi	= 38;
const uint8_t gpio_pin_rst = 42;

// Initialise the GPIO for bitbanging the SPI
void SPI_Init(void) {
	bcm2835_gpio_fsel(gpio_pin_cs, BCM2835_GPIO_FSEL_OUTP);		
	bcm2835_gpio_set(gpio_pin_cs);	// CS is active low, so set CS high to start
	bcm2835_gpio_fsel(gpio_pin_clk, BCM2835_GPIO_FSEL_OUTP);	
	bcm2835_gpio_set(gpio_pin_clk);	// CLK should idle high according to ILI9481 datasheet						
	bcm2835_gpio_fsel(gpio_pin_mosi, BCM2835_GPIO_FSEL_OUTP);	
	bcm2835_gpio_clr(gpio_pin_mosi);
	bcm2835_gpio_fsel(gpio_pin_mosi, BCM2835_GPIO_FSEL_OUTP);	
	bcm2835_gpio_set(gpio_pin_rst);
}

// Assert the CS line to the ILI9481 (pull it low)
void LCD_SPI_CS(uint8_t cs_status) {
	if(cs_status)
		bcm2835_gpio_set(gpio_pin_cs);
	else
		bcm2835_gpio_clr(gpio_pin_cs);
}

void LCD_Rst(void){			
    bcm2835_gpio_clr(gpio_pin_rst);
    bcm2835_delay(20);					   
    bcm2835_gpio_set(gpio_pin_rst);	 	 
    bcm2835_delay(20);	
}

void LCD_WriteByteSPI(unsigned char value){
   int8_t n;
   
   for(n=7; n>=0; n--)			
   {  
   	bcm2835_delayMicroseconds(SPI_HOLDTIMEUS);
		bcm2835_gpio_clr(gpio_pin_clk);		// Pull the clock low.
		bcm2835_gpio_write(gpio_pin_mosi, (value >> n) & 1);	// Setup the data line
		bcm2835_delayMicroseconds(SPI_HOLDTIMEUS);
		bcm2835_gpio_set(gpio_pin_clk);		// Push the clock high. This will clock the value into the ILI9841
   }
}

void SPI_WriteComm(uint8_t CMD)
{			
	LCD_SPI_CS(0);
	LCD_WriteByteSPI(0x70);
	LCD_WriteByteSPI(CMD);	
	LCD_SPI_CS(1);
}

void SPI_WriteData(uint8_t tem_data)
{			
	LCD_SPI_CS(0);
	LCD_WriteByteSPI(0x72);
	LCD_WriteByteSPI(tem_data);
	LCD_SPI_CS(1);
}

int main()  
{    
   if (!bcm2835_init())  
       return 1;  
    
   SPI_Init();
    
   LCD_SPI_CS(0);
   LCD_Rst();
   LCD_SPI_CS(1);
   
   printf("IO configure finish\n");
    
   SPI_WriteComm(0x20);//exit_invert_mode
   SPI_WriteComm(0x29);//set_display_on
   SPI_WriteComm(0x3A);//set_pixel_format 
   SPI_WriteData(0x77);//70   0X60 26k
   printf("Configured reg 0x20 \n");
   
   SPI_WriteComm(0xB1);//RGB Interface Setting
   SPI_WriteData(0x00);//VHSYNC Polarity
   SPI_WriteData(0x14);
   SPI_WriteData(0x06);
   printf("Configured reg 0xB1 \n");
 
   SPI_WriteComm(0xB2);//Panel Characteristics Setting
   SPI_WriteData(0x10);//480 pixels
   SPI_WriteData(0xC8);//800 pixels
   printf("Configured reg 0xB3 \n");
 
   SPI_WriteComm(0xB3);//Panel Drive Setting    Set the inversion mode
   SPI_WriteData(0x00);//1-dot inversion 0x01
   printf("Configured reg 0xB3 \n");
 
   SPI_WriteComm(0xB4);//Display Mode Control
   SPI_WriteData(0x04);//Dither disable.
   printf("Configured reg 0xB4 \n");
 
   SPI_WriteComm(0xB5);//Display Mode and Frame Memory Write Mode Setting
   SPI_WriteData(0x10);
   SPI_WriteData(0x30);
   SPI_WriteData(0x30);
   SPI_WriteData(0x00);
   SPI_WriteData(0x00);
   printf("Configured reg 0xB5 \n");
 
   SPI_WriteComm(0xB6);//Display Control 2 ( GIP Specific )
   SPI_WriteData(0x01);
   SPI_WriteData(0x18);
   SPI_WriteData(0x02);
   SPI_WriteData(0x40);
   SPI_WriteData(0x10);
   SPI_WriteData(0x00);
   printf("Configured reg 0xB6 \n");
 
   SPI_WriteComm(0xC0);
   SPI_WriteData(0x00);//Enable Internal OSC
   SPI_WriteData(0x00);//0x18
   printf("Configured reg 0xC0 \n");
 
   SPI_WriteComm(0xC3); 
   SPI_WriteData(0x03);
   SPI_WriteData(0x04);
   SPI_WriteData(0x03);
   SPI_WriteData(0x03);
   SPI_WriteData(0x03);
   printf("Configured reg 0xC3 \n");
   
   bcm2835_delay(1);
 
   SPI_WriteComm(0xC4);//VDD Regulator setting
   SPI_WriteData(0x02);
   SPI_WriteData(0x23);//GDC AP
   SPI_WriteData(0x11);//VRH1  Vreg1out=1.533xVCI(10)
   SPI_WriteData(0x12);//VRH2  Vreg2out=-1.533xVCI(10)
   SPI_WriteData(0x02);//BT 5 VGH/VGL  6/-4
   SPI_WriteData(0x77);//DDVDH 6C//0x56
   printf("Configured reg 0xC4 \n");
   
   bcm2835_delay(1);
 
   SPI_WriteComm(0xC5);
   SPI_WriteData(0x73);
   printf("Configured reg 0xC5 \n");
   
   bcm2835_delay(1);
 
   SPI_WriteComm(0xC6);
   SPI_WriteData(0x24);//VCI 23
   SPI_WriteData(0x60);//RESET RCO 53
   SPI_WriteData(0x00);//SBC GBC
   printf("Configured reg 0xC6 \n");
   
   bcm2835_delay(1);
    //GAMMA SETTING
   SPI_WriteComm(0xD0);
   SPI_WriteData(0x14);
   SPI_WriteData(0x01);
   SPI_WriteData(0x53);
   SPI_WriteData(0x25);
   SPI_WriteData(0x02);
   SPI_WriteData(0x02);
   SPI_WriteData(0x66);
   SPI_WriteData(0x14);
   SPI_WriteData(0x03);
   printf("Configured reg 0xD0 \n");
 
   SPI_WriteComm(0xD1);
   SPI_WriteData(0x14);
   SPI_WriteData(0x01);
   SPI_WriteData(0x53);
   SPI_WriteData(0x07);
   SPI_WriteData(0x02);
   SPI_WriteData(0x02);
   SPI_WriteData(0x66);
   SPI_WriteData(0x14);
   SPI_WriteData(0x03);
   printf("Configured reg 0xD1 \n");
 
   SPI_WriteComm(0xD2);
   SPI_WriteData(0x14);
   SPI_WriteData(0x01);
   SPI_WriteData(0x53);
   SPI_WriteData(0x25);
   SPI_WriteData(0x02);
   SPI_WriteData(0x02);
   SPI_WriteData(0x66);
   SPI_WriteData(0x14);
   SPI_WriteData(0x03);
   printf("Configured reg 0xD2 \n");
 
   SPI_WriteComm(0xD3);
   SPI_WriteData(0x14);
   SPI_WriteData(0x01);
   SPI_WriteData(0x53);
   SPI_WriteData(0x07);
   SPI_WriteData(0x02);
   SPI_WriteData(0x02);
   SPI_WriteData(0x66);
   SPI_WriteData(0x14);
   SPI_WriteData(0x03);
   printf("Configured reg 0xD3 \n");
  
   SPI_WriteComm(0xD4);
   SPI_WriteData(0x14);
   SPI_WriteData(0x01);
   SPI_WriteData(0x53);
   SPI_WriteData(0x25);
   SPI_WriteData(0x02);
   SPI_WriteData(0x02);
   SPI_WriteData(0x66);
   SPI_WriteData(0x14);
   SPI_WriteData(0x03);
   printf("Configured reg 0xD4 \n");
 
   SPI_WriteComm(0xD5);
   SPI_WriteData(0x14);
   SPI_WriteData(0x01);
   SPI_WriteData(0x53);
   SPI_WriteData(0x07);
   SPI_WriteData(0x02);
   SPI_WriteData(0x02);
   SPI_WriteData(0x66);
   SPI_WriteData(0x14);
   SPI_WriteData(0x03);
   SPI_WriteComm(0x11);
   printf("Configured reg 0xD1 \n");

   bcm2835_delay(1);
   SPI_WriteComm(0x3A); SPI_WriteData(0x66);
   SPI_WriteComm(0x36); SPI_WriteData(0x00);
   //SPI_WriteComm(0x29);
   printf("LCD initial complate! \n");
  
   return 0;  
}  
